/* crash.c - part of crash
 * Copyright (C) 2000 Tachibana Eriko
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

/****************************************************************
*	vAu						*
****************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <signal.h>
#include <unistd.h>
#include <a_out.h>
#include <sys/xstart.h>
#include "crash.h"

/****************************************************************
*	ϐ							*
****************************************************************/

int flagVerbose = 1;				/* verbose \x */
char *coreFname;				/* RAC[Wt@C */
char *execFname;				/* st@C */

/****************************************************************
*	gp@̕\						*
****************************************************************/

static void Usage (void)
{
    /* gp@\ */
    fprintf (stderr,
	     "gp@: crash [IvV] exec_file core_file\n"
	     "\t-q\t璷ȃbZ[Wo͂Ȃ\n"
	     "\t-v<n>\tbZ[W\x <n> ɐݒ肷\n");
}

/****************************************************************
*	VOinh					*
****************************************************************/

static void SignalHandler (int signo)
{
    /* VOiɂ... */
    switch (signo) {

    case SIGINT:
	/* SIGINT Ȃ... */
	printf ("\nI\n\n");
	break;

    default:
	/* ̑̃VOi */
	printf ("\nVOi %d '%s' ɂĒf܂\n\n", signo, strsignal (signo));
	break;

    }

    /* I */
    exit (2);
}

/****************************************************************
*	R}hC					*
****************************************************************/

int PromptInput (const char *prompt, int *acp, char ***avp)
{
    int i, argc;
    char buffer[256];
    char tokens[256];
    char *p;
    static char *argv[MAX_PARGS];

    /* vvg\ */
    if (isatty (STDIN_FILENO))
	printf (prompt);

    /* s͂s */
    if (fgets (buffer, sizeof buffer, stdin) == NULL)
	return EOF;
    if ((p = strchr (buffer, '\n')) != NULL)
	*p = '\0';

    /* R}hCg[Nɕ */
    argc = _dehupair (buffer, tokens);

    /* g[No */
    for (i = 0, p = tokens; i < argc; i++) {

	/* zɋL^ */
	argv[i] = p;

	/* ̃g[N̓ */
	while (*p++);

    }

    /* ̐L^ */
    *acp = argc;

    /* z̃AhXL^ */
    *avp = argv;

    /* I */
    return 0;
}

/****************************************************************
*	C[`						*
****************************************************************/

int main (int argc, char **argv)
{
    int ch, ac;
    char **av;

    /* VOinhݒ */
    signal (SIGINT , SignalHandler);
    signal (SIGBUS , SignalHandler);
    signal (SIGSEGV, SignalHandler);
    signal (SIGILL , SignalHandler);
    signal (SIGEMT , SignalHandler);

    /* IvV */
    while ((ch = getopt (argc, argv, "qv:")) != EOF) {

	/* IvVɂĕ */
	switch (ch) {

	case 'q':
	    /* Quiet [h */
	    flagVerbose = 0;
	    break;

	case 'v':
	    /* Verbose xݒ */
	    flagVerbose = atoi (optarg);
	    break;

	default:
	    /* sȃIvV */
	    Usage ();
	    return EXIT_FAILURE;

	}

    }

    /* ̐`FbN */
    if (argc - optind != 2) {
	Usage ();
	return EXIT_FAILURE;
    }

    /* V{e[uǂݍ */
    execFname = argv[optind];
    ReadSymbols (execFname);

    /* RAC[Wǂݍ */
    coreFname = argv[optind + 1];
    ReadCore (coreFname);

    /* L[ */
    while (1) {

	/* R}h͂A͂ */
	if (PromptInput ("crash> ", &ac, &av) == EOF)
	    break;

	/* 邩? */
	if (ac > 0) {
	    void (*func) (int, char **) = NULL;

	    /* R}h */
	    if (strcmp (av[0], "fd") == 0)
		func = ShowFileHandle;

	    else if (strcmp (av[0], "help") == 0)
		func = ShowHelp;

	    else if (strcmp (av[0], "pos") == 0)
		func = ShowPosition;

	    else if (strcmp (av[0], "proc") == 0)
		func = ShowProcess;

	    else if (strcmp (av[0], "quit") == 0)
		func = ShowQuit;

	    else if (strcmp (av[0], "stack") == 0)
		func = ShowStack;

	    else if (strcmp (av[0], "str") == 0)
		func = ShowStreams;

	    /* Tu[`s */
	    if (func)
		(*func) (ac, av);

	}

    }

    /* IR[h 0 */
    return EXIT_SUCCESS;
}
